
// Layout variables 
int margin = 20;
float plotW = width - margin*2;
float plotH = height - margin*2;

// interaction variables
String x_variable = "age", y_variable = "full_frequency", color_representation = "bookType";
float hoverDistance = plotW;

float max_age = 0, max_relative_age = 0, max_fullFrequency = 0, max_partialFrequency = 0;
float min_age = 10000, min_relative_age = 10000, min_fullFrequency = 10000, min_partialFrequency = 10000;
int min_publishOrder = 1000, max_publishOrder = 0;

float valX = 0, valY = 0, start1_X = 0, stop1_X = 0, start2_X = 0, stop2_X = 0, start1_Y = 0, stop1_Y = 0, start2_Y = 0, stop2_Y = 0;



Table table;
int numRows, numCols;

DataPoint[] dataPoints;

String publishDate,itemType;
int bib, checkOutCount, adoption, age, relative_age, publishOrder;
String minCheckOut, maxCheckOut;
float fullFrequency, partialFrequency;

void setup() {
  size(1440, 900);
  smooth();
  //frameRate(60);
  surface.setResizable(true); // make the screen resizable
  
  table = loadTable("final_cleaned.csv", "header"); // put data into Processing table from csv
  numRows = table.getRowCount(); // get number of data points
  numCols = table.getColumnCount(); // get number of attributes per data point
  
  // create data points and fill array
  dataPoints = new DataPoint[numRows+1]; // initialize data points array length
  int counter = 0;
  for (TableRow row : table.rows()){
    counter +=1;
    createDataPoint(row, counter); 
  }
}

void draw() {
  background(50);
  
  for(int i=1; i<dataPoints.length; i++){
    checkXVariable(i);
    checkYVariable(i);
    // //checkColor(i);
    // map values to x and y axis
    float posX = map(valX, start1_X, stop1_X, start2_X, stop2_X);
    float posY = map(valY, start1_Y, stop1_Y, start2_Y, stop2_Y);
    
    if(mouseX >= posX - hoverDistance && mouseX <= posX + hoverDistance  && mouseY >= posY - hoverDistance && mouseY <= posY + hoverDistance){
      dataPoints[i].displayLabel();
      if(color_representation == "bookType"){
        String bookType = dataPoints[i].itemType;
        dataPoints[i].setColor(i, bookType);
      }
    } else { dataPoints[i].setColor(i, "null"); }
    
    dataPoints[i].setPosition(posX, posY);
    dataPoints[i].display();
  }
  
  drawBoarders();
  drawLabels();
  drawTitles();
  drawAxis();
  drawInformation();
}

void drawBoarders(){
  fill(100);
  rect(0,0,margin,height);
  rect(0,0,width,margin);
  rect(0,height-margin,width,margin);
  rect(width-margin,margin,margin,height);
}

void drawLabels(){
  rect(0,0, margin, height);
  fill(50);
}

void drawTitles(){
  textAlign(LEFT, CENTER);
  textSize(12);
  fill(200);
  text("Book Age (days)", width/2, .5*margin);
  text("Checkout Frequency (occurances)", margin*2, height/2);
}

void drawAxis(){
  float x_axis_distance = (stop2_X - start2_X) / 10;
  float y_axis_distance = (start2_Y - stop2_Y) / 10;
  
  float x_axis_value = (stop1_X - start1_X) / 10;
  float y_axis_value = (stop1_Y - start1_Y) / 10;
  
  // print x axis
  for(int k=1; k<11; k++){
    textAlign(CENTER, CENTER);
    textSize(8);
    fill(200);
    text(k*x_axis_value, k*x_axis_distance, margin*2);
  }
  
  // print y axis
  for(int l=1; l<11; l++){
    textAlign(CENTER, CENTER);
    textSize(8);
    fill(200);
    text(l*y_axis_value, margin+20, height - (l*y_axis_distance));
  }
  
}

void drawInformation(){
  textSize(20);
  rotate(.5);
  translate(-100, -1*height/1.5 + 50);
  textAlign(CENTER, CENTER);
  text("Are Books Still Prevelant in an e-Media World?", 5*(width/6), margin*6);
  textSize(12);
  
  if(x_variable.equals("age")){ text("X-Variable: Age", 5*(width/6), margin*7); }
  else {text("X_Variable: Relative Age", 5*(width/6), margin*7);}
  
  if(y_variable.equals("full_frequency")){ text("Y-Variable: Checkout Frequency", 5*(width/6), margin*8); }
  else {text("Y_Variable: Relative Checkout Frequency", 5*(width/6), margin*8);}
  
  if(color_representation.equals("bookType")){ text("Color Range: Book Type", 5*(width/6), margin*9); }
  else{ text("Color Range: Publication Date", 5*(width/6), margin*9); }
}